@@ -6,6 +6,7 @@ import pytest
 # TODO this should not be needed if the tuxmake RPM manages to provide a public
 # Python module that is not tied to a specific Python version
 sys.path.append("/usr/share/tuxmake")
+from tuxmake.runtime import Runtime
 
 
 oebuild_git_params = {
@@ -174,3 +175,14 @@ def oebuild_repo_init_object():
 
     oebuild = OEBuild(**oebuild_repo_object)
     return oebuild
+
+
+@pytest.fixture()
+def metadata_obj(oebuild_repo_init_object):
+    from tuxbake.metadata import Metadata
+
+    oebuild = oebuild_repo_init_object
+    # prepare runtime
+    runtime = Runtime.get(oebuild.runtime)
+    oebuild._runtime = runtime
+    return Metadata(oebuild)
