@@ -2,15 +2,27 @@
 # -*- coding: utf-8 -*-
 
 import argparse
-from tuxmake.runtime import Runtime
-from typing import List
 import os
+from typing import List
+from urllib.parse import urlparse
+
+from tuxbake.utils import download_file
+from tuxmake.runtime import Runtime
 
 
 class supported:
     runtimes: List[str] = Runtime.supported()
 
 
+def file_or_url(value):
+    if os.path.exists(value):
+        return value
+    elif urlparse(value).scheme in ["http", "https"]:
+        return download_file(value, os.path.abspath("build-definition.json"))
+
+    raise argparse.ArgumentTypeError(f"'{value}' is not a valid file path or URL")
+
+
 ##########
 # Setups #
 ##########
@@ -22,8 +34,9 @@ def setup_parser() -> argparse.ArgumentParser:
     group = parser.add_argument_group("OE Build Parameters")
     group.add_argument(
         "--build-definition",
-        help="Specify json file with build parameters",
+        help="Path/URL to build definition json file",
         required=True,
+        type=file_or_url,
     )
     group.add_argument(
         "--build-only",
@@ -63,6 +76,11 @@ def setup_parser() -> argparse.ArgumentParser:
         help="Directory name passed to the source script during OE build. It defaults to build",
         default="build",
     )
+    group.add_argument(
+        "--home-dir",
+        help="Path to the home directory to be used as home inside the container. This can be used to pass credentials.",
+        default=None,
+    )
     group.add_argument(
         "--local-manifest",
         help="Path to a local manifest file which will be used during repo sync. This input is ignored if sources used is git_trees in the build definition",
