@@ -2,8 +2,10 @@
 # -*- coding: utf-8 -*-
 
 from tuxbake.models import OEBuild
+from tuxbake.metadata import Metadata
 from tuxmake.runtime import Terminated
 import signal
+import sys
 
 
 def build(**kwargs):
@@ -14,8 +16,16 @@ def build(**kwargs):
         oebuild.do_build()
     except (KeyboardInterrupt, Terminated):
         print("tuxbake Interrupted")
+
     if oebuild.artifacts_dir:
         oebuild.publish_artifacts()
+    # try to collect metadata
+    try:
+        metadata = Metadata(oebuild)
+        metadata.collect()
+    except Exception as ex:
+        sys.stderr.write(f"{str(ex)}\n")
+
     oebuild.do_cleanup()
     signal.signal(signal.SIGTERM, old_sigterm)
     return oebuild
