@@ -18,3 +18,13 @@ class TuxbakeParsingError(TuxbakeError):
 class TuxbakeValidationError(TuxbakeError):
     error_help = "Error while validating API arguments"
     error_type = "Validation"
+
+
+class UnsupportedMetadata(TuxbakeParsingError):
+    error_help = "Unsupported metadata"
+    error_type = "Parsing"
+
+
+class UnsupportedMetadataType(TuxbakeParsingError):
+    error_help = "Unsupported metadata type: {name}"
+    error_type = "Validation"
