@@ -6,7 +6,13 @@ from dataclasses import asdict, dataclass, field, fields
 from typing import Dict, List, Optional
 from tuxmake.runtime import Runtime
 from tuxmake.logging import set_debug, debug
-from tuxbake.utils import repo_init, git_init, copy_artifacts
+from tuxbake.utils import (
+    repo_init,
+    git_init,
+    copy_artifacts,
+    get_filtered_paths,
+    find_bitbake_env,
+)
 from pathlib import Path
 from tuxbake.exceptions import TuxbakeParsingError
 from tuxbake.helper import (
@@ -51,6 +57,7 @@ class OEBuild(Base):
     build_dir: str
     envsetup: Optional[str] = None
     targets: Optional[List[str]] = None
+    after_cmds: Optional[List[str]] = None
     extraconfigs: Optional[List[str]] = None
     extraconfigs_cmd: str = ""
     distro: Optional[str] = None
@@ -77,6 +84,7 @@ class OEBuild(Base):
     # For future use
     artifacts: Optional[List[str]] = None
     artifacts_dir: Optional[str] = None
+    home_dir: str = None
 
     @dataclass
     class Repo:
@@ -103,7 +111,6 @@ class OEBuild(Base):
 
     @staticmethod
     def validate(oebuild_data):
-
         if not oebuild_data.get("sources"):
             raise (
                 TuxbakeParsingError(
@@ -285,7 +292,7 @@ class OEBuild(Base):
         debug(f"build-only flag set to: {self.build_only}")
         debug(f"sync-only flag set to: {self.sync_only}")
         if self.build_only and self.sync_only:
-            print("ERROR: Both build-only and sync-only shoun't be set.")
+            print("ERROR: Both build-only and sync-only shouldn't be set.")
         if self.build_only:
             return
         if self.sources.get("repo"):
@@ -314,6 +321,8 @@ class OEBuild(Base):
         self._runtime.output_dir = Path(self.log_dir)
         if self.dl_dir:
             self._runtime.add_volume(self.dl_dir)
+        if self.home_dir:
+            self._runtime.add_volume(self.home_dir, "/home/tuxbake")
         if self.sstate_dir and not self.sstate_dir.startswith(self.src_dir):
             self._runtime.add_volume(self.sstate_dir)
         environment = self.environment
@@ -330,7 +339,7 @@ class OEBuild(Base):
             if self.sstate_dir:
                 extra_local_conf.write(f'SSTATE_DIR = "{self.sstate_dir}"\n')
             if self.sstate_mirror:
-                extra_local_conf.write(f'SSTATE_MIRRORS ?= "{self.sstate_mirror}"\n')
+                extra_local_conf.write(f'SSTATE_MIRRORS = "{self.sstate_mirror}"\n')
                 extra_local_conf.write(
                     'USER_CLASSES += "buildstats buildstats-summary"\n'
                 )
@@ -360,7 +369,7 @@ class OEBuild(Base):
         cmd = [
             "bash",
             "-c",
-            f"rm -f {self.build_dir}/conf/bblayers.conf && rm -f {self.build_dir}/conf/local.conf && source {self.envsetup} {self.build_dir} {self.extraconfigs_cmd} && cat ../extra_local.conf >> conf/local.conf && cat ../bblayers.conf >> conf/bblayers.conf || true && echo 'Dumping local.conf..' && cat conf/local.conf && bitbake -e > bitbake-environment",
+            f"rm -f {self.build_dir}/conf/bblayers.conf && rm -f {self.build_dir}/conf/local.conf && source {self.envsetup} {self.build_dir} {self.extraconfigs_cmd} && cat ../extra_local.conf >> conf/local.conf && cat ../bblayers.conf >> conf/bblayers.conf || true && echo 'Dumping local.conf..' && cat conf/local.conf",
         ]
         if self._runtime.run_cmd(cmd, offline=False):
             self.result = "pass"
@@ -368,6 +377,14 @@ class OEBuild(Base):
             self.result = "fail"
             return
 
+        # Dump bitbake -e separately
+        cmd = [
+            "bash",
+            "-c",
+            f"source {self.envsetup} {self.build_dir} && bitbake -e > bitbake-environment",
+        ]
+        self._runtime.run_cmd(cmd, offline=False)
+
         for target in self.targets:
             cmd = [
                 "bash",
@@ -380,15 +397,36 @@ class OEBuild(Base):
                 self.result = "fail"
                 return
 
+        if not self.after_cmds:
+            return
+
+        for cmd in self.after_cmds:
+            cmd = [
+                "bash",
+                "-c",
+                f"source {self.envsetup} {self.build_dir} && {cmd}",
+            ]
+            if self._runtime.run_cmd(cmd, offline=False):
+                self.result = "pass"
+            else:
+                self.result = "fail"
+                return
+
     def publish_artifacts(self):
-        if self.distro.startswith("openbmc"):
-            bitbake_env_file = (
-                f"{self.src_dir}/openbmc/{self.build_dir}/bitbake-environment"
-            )
-        else:
-            bitbake_env_file = f"{self.src_dir}/{self.build_dir}/bitbake-environment"
-        artifacts_dir = f"{self.src_dir}/{self.artifacts_dir}"
-        copy_artifacts(self.artifacts, self.src_dir, artifacts_dir, bitbake_env_file)
+        build_dir = f"{self._runtime.source_dir}/{self.build_dir}"
+        bitbake_env_file = f"{build_dir}/bitbake-environment"
+        deploy_dir = find_bitbake_env(bitbake_env_file, "DEPLOY_DIR")
+        artifacts_path_list = [bitbake_env_file]
+        if self.artifacts:
+            build_dir_paths = get_filtered_paths(self.artifacts, build_dir)
+            deploy_dir_paths = get_filtered_paths(self.artifacts, deploy_dir)
+            artifacts_path_list += build_dir_paths + deploy_dir_paths
+        elif deploy_dir is not None:
+            # whole DEPLOY_DIR to be published
+            artifacts_path_list += [f"{deploy_dir}/."]
+
+        dest_dir = f"{self.src_dir}/{self.artifacts_dir}"
+        copy_artifacts(artifacts_path_list, dest_dir)
 
     def do_cleanup(self):
         if self._runtime:
