@@ -1,10 +1,14 @@
 import os
+import json
 import sys
 import subprocess
+import glob
 from pathlib import Path
+import requests
 from tuxbake.exceptions import TuxbakeRunCmdError, TuxbakeParsingError
 from tuxmake.logging import debug
 import logging
+from retrying import retry
 
 formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
 logger = logging.getLogger(__name__)
@@ -35,6 +39,12 @@ def log_handler(func):
 
 
 @log_handler
+@retry(
+    wait_exponential_multiplier=500,
+    wait_exponential_max=10000,
+    stop_max_attempt_number=3,
+    retry_on_exception=lambda e: isinstance(e, TuxbakeRunCmdError),
+)
 def repo_init(oebuild, src_dir, local_manifest=None, pinned_manifest=None):
     cmd = f"repo init -u {oebuild.repo.url} -b {oebuild.repo.branch} -m {oebuild.repo.manifest}".split()
     run_cmd(cmd, src_dir)
@@ -54,7 +64,7 @@ def repo_init(oebuild, src_dir, local_manifest=None, pinned_manifest=None):
 
 
 def git_fetch(what_to_fetch, src_dir, dir_path):
-    cmd = f"git fetch origin --quiet {what_to_fetch}".split()
+    cmd = f"git fetch origin --quiet --recurse-submodules=no {what_to_fetch}".split()
     run_cmd(cmd, os.path.join(src_dir, dir_path))
 
 
@@ -69,11 +79,18 @@ def git_get_sha(branch, url):
 
 
 @log_handler
+@retry(
+    wait_exponential_multiplier=500,
+    wait_exponential_max=10000,
+    stop_max_attempt_number=3,
+    retry_on_exception=lambda e: isinstance(e, TuxbakeRunCmdError),
+)
 def git_init(oebuild, src_dir):
     for git_object in oebuild.git_trees:
         url = git_object.url.rstrip("/")
         branch = git_object.branch or git_object.ref
         sha = git_object.sha or git_get_sha(branch, url)
+        git_object.sha = sha
         dest = git_object.dest
         if not (branch or sha):
             raise TuxbakeRunCmdError(
@@ -110,41 +127,52 @@ def git_init(oebuild, src_dir):
         cmd = f"git checkout {sha}".split()
         run_cmd(cmd, os.path.join(src_dir, dir_path))
 
+        cmd = f"git submodule update --init --recursive".split()
+        run_cmd(cmd, os.path.join(src_dir, dir_path))
+
+
+def get_filtered_paths(artifacts, path):
+    """
+    Return resolved artifacts paths which are present relative to provided path. Rest other paths are discarded.
+    """
+    dir_paths = []
+    if artifacts and path:
+        for artifact in artifacts:
+            resolved_path = str(Path(f"{path}/{artifact}").resolve())
+            path_files = glob.glob(resolved_path)
+            if path_files and (resolved_path + os.sep).startswith(path + os.sep):
+                dir_paths += path_files
+
+    return dir_paths
+
 
 def find_bitbake_env(env_file, key):
+    if not os.path.exists(env_file):
+        return
     with open(env_file) as env:
         for line in env.readlines():
             if line.startswith(f"{key}="):
                 return line.split("=")[1].strip().replace('"', "")
 
 
-def copy_artifacts(artifacts, src_dir, artifacts_dir, bitbake_env_file):
-    if not os.path.exists(bitbake_env_file):
-        return
-    deploy_dir = find_bitbake_env(bitbake_env_file, "DEPLOY_DIR")
-    if deploy_dir:
-        if artifacts:
-            dir_paths = [
-                str(Path(f"{deploy_dir}/{artifact}").resolve())
-                for artifact in artifacts
-            ]
-            dirs = " ".join(dir_paths)
-        else:
-            # whole DEPLOY_DIR to be published
-            dirs = f"{deploy_dir}/."
-
-        if not os.path.exists(artifacts_dir):
-            os.makedirs(artifacts_dir)
-        else:
-            cmd = f"rm -rf {artifacts_dir}".split()
-            run_cmd(cmd, src_dir)
-            cmd = f"mkdir -p {artifacts_dir}".split()
-            run_cmd(cmd, src_dir)
-        cmd = f"cp -R {dirs} {artifacts_dir}".split()
-        run_cmd(cmd, src_dir, fail_ok=True)
-        # copy bitbake-environment files as well
-        cmd = f"cp -R {bitbake_env_file} {artifacts_dir}".split()
-        run_cmd(cmd, src_dir, fail_ok=True)
+def copy_artifacts(artifacts_path_list, dest_dir):
+    # artifacts: list of paths
+    dirs = " ".join(
+        [
+            path
+            for path in artifacts_path_list
+            if path is not None and isinstance(path, str)
+        ]
+    )
+    if not os.path.exists(dest_dir):
+        os.makedirs(dest_dir)
+    else:
+        cmd = f"rm -rf {dest_dir}".split()
+        run_cmd(cmd, None)
+        cmd = f"mkdir -p {dest_dir}".split()
+        run_cmd(cmd, None)
+    cmd = f"cp -R {dirs} {dest_dir}".split()
+    run_cmd(cmd, None, fail_ok=True)
 
 
 def run_cmd(cmd, src_dir, env=None, fail_ok=False):
@@ -176,3 +204,19 @@ def handle_log(logs_list):
             else:
                 logger.info(data)
     return
+
+
+def download_file(url, file_dest):
+    try:
+        response = requests.get(url)
+        response.raise_for_status()
+        json_data = response.json()
+        with open(file_dest, "w") as build_definition:
+            json.dump(json_data, build_definition, indent=4)
+        return file_dest
+    except requests.exceptions.JSONDecodeError as e:
+        print(f"Invalid json file: {e}")
+        sys.exit(1)
+    except requests.exceptions.RequestException as e:
+        print(f"Failed to download build definition file: {e}")
+        sys.exit(1)
